/**
 * CeboMsr - C++ example - Info.
 *
 * Print out various information about the device.
 *
 * (c) 2013 Cesys GmbH
 * @author Thomas Hoppe
 */

#include <iostream>
#include <stdexcept>

#include <cebomsrpp.h>

using namespace std;
using namespace CeboMsr;

/**
 * Print various information to the console.
 */
void printInformation(Device &device) {
	// Put out some device specific information.
	cout << "Device type: " << device.getDeviceType().getName() << endl;
	cout << "USB base ver: " << LibraryInterface::getUsbBaseVersion() << endl;
	cout << "API vers: " << LibraryInterface::getApiVersion() << endl;
	cout << "Firmware ver: " << device.getFirmwareVersion() << endl;
	cout << "Identifier: " << device.getIdentifier() << endl;
	cout << "Serial number: " << device.getSerialNumber() << endl;
	cout << "Temperature: " << device.getTemperature() << endl;

	// Get the real current of the reference current sources.
	for (size_t i = 0; i < device.getCurrentSources().size(); ++i) {
		CurrentSource source = device.getCurrentSources().at(i);
		cout << "Reference current of "
			<< source.getName() << ": "
			<< source.getReferenceCurrent() << " uA" << endl;
	}

	// Retrieve information about single ended input #0.
	// This works on all analog inputs and outputs.
	AnalogInput se0 = device.getSingleEndedInputs().at(0);
	cout << "Info for single ended input " << se0.getName() << ":" << endl;
	cout << "Min. ICD: " << se0.getMinInterChannelDelay() << " us" << endl;
	cout << "  Current range: "
		<< se0.getRange().getMinValue() << " V to "
		<< se0.getRange().getMaxValue() << " V" << endl;
	cout << "  Supported ranges:" << endl;
	for (size_t i = 0; i < se0.getSupportedRanges().size(); ++i) {
		Range range = se0.getSupportedRanges().at(i);
		cout << "    Range #" << i << " range: "
			<< range.getMinValue() << " V to "
			<< range.getMaxValue() << " V, "
			<< "Def. ICD: "
			<< se0.getDefaultInterChannelDelay(range) << " us"
			<< endl;
	}

	// Get info for digital port #0.
	DigitalPort dp0 = device.getDigitalPorts().at(0);
	cout << "Info for digital port " << dp0.getName() << ":" << endl;
	cout << "  Count of I/O's: " << dp0.getIoCount() << endl;
	for (int i = 0; i < dp0.getIoCount(); ++i) {
		cout << "    I/O #" << i << ":" << dp0.getIoName(i) << endl;
	}
}

/**
 * Application entry point.
 */
int main() {
	// Nearly all methods can raise exceptions. For this example, the error handling is done at this single point.
	try {
		// Search for devices ...
		DeviceVector devices = LibraryInterface::enumerate(DeviceType::All);

		// If at least one has been found, use the first one ...
		if (!devices.empty()) {
			Device device = devices[0];

			// Open device, nothing can be done without doing this.
			device.open();

			// Print some information to the console.
			printInformation(device);

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			device.close();
		}
	} // These exceptions are currently used to report different problems ...
	catch (const std::invalid_argument &e) { cout << "Invalid argument exception raised: " << e.what() << endl; }
	catch (const std::out_of_range &e) { cout << "Out of range exception raised: " << e.what() << endl; }
	catch (const std::logic_error &e) { cout << "Logic error exception raised: " << e.what() << endl; }
	catch (const std::runtime_error &e) { cout << "Runtime/IO error exception raised: " << e.what() << endl; }
	catch (const std::exception &e) { cout << "Unknown exception raised: " << e.what() << endl; }

	return 0;
}
