﻿using System;
using System.Collections.Generic;
using CeboMsrNet;
using System.IO;

/**
 * CeboMsr - C# example - Single I/O.
 *
 * The I/O's of digital port #0 are read and mirrored to digital port #1.
 * (IO-0 - IO-7 are mirrored to IO-8 - IO-15).
 * The voltage at AI-0 is mirrored to AO-1 and AI-1 to AO-0 too.
 *
 * Does only work on CeboLC devices.
 * 
 * @author Thomas Hoppe
 */
namespace SingleIoExample
{
    class SingleIoExample
    {
        /// <summary>
        /// Do very simple IO, digital as well as analog.
        /// </summary>
        /// <param name="device">Device to use for test.</param>
        private void SingleIoTest(Device device)
        {
            // Read digital port #0 and write result to digital port #1 (mirror pins).
            // At first, we need references to both ports.
            DigitalPort dp0 = device.DigitalPorts[0];
            DigitalPort dp1 = device.DigitalPorts[1];

            // Configure port first, all bits of digital port #0 as input (default)
            // and all bits of digital port #1 as output.
            dp1.OutputEnableMask = 0xff;

            // Read from #0 ...
            int value = dp0.Value;

            // Write the value to #1 ...
            dp1.Value = value;

            // Now some analog I/O, do it without any additional local references.
            // Read single ended #0 input voltage ...
            float voltageValue = device.SingleEndedInputs[0].Value;

            // Write value to analog output #1
            device.AnalogOutputs[1].Value = voltageValue;

            // .. or in one line: single ended #1 to analog output #0 ...
            device.AnalogOutputs[0].Value = device.SingleEndedInputs[1].Value;

            // Reset to power up defaults.
            device.ResetDevice();
        }

        /// <summary>
        /// The examples main method.
        /// </summary>
        private void RunExample()
        {
            try
            {
                // Search for devices ...
                IList<Device> devices = LibraryInterface.Enumerate(DeviceType.CeboLC);

                // If at least one has been found, use the first one ...
                if (devices.Count > 0)
                {
                    Device device = devices[0];

                    // Open device, nothing can be done without doing this.
                    device.Open();

                    // *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
                    // *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
                    // *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
                    SingleIoTest(device);

                    // Finalize device usage, this free's up the device, so it can be used
                    // again, including other applications.
                    device.Close();
                }
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        static void Main(string[] args)
        {
            new SingleIoExample().RunExample();
        }
    }
}
