﻿// (c) 2013 Cesys GmbH
// Author: Thomas Hoppe
using System;
using System.Collections.Generic;
using System.Text;

namespace CeboMsrNet
{
    /// <summary>
    /// Offers functionality of the different kind of analog inputs.
    /// Direct construction is not possible, instances can be retrieved from
    /// SingleEndedInputs or DifferentialInputs of the respective instance
    /// of class Device.
    /// </summary>
    public sealed class AnalogInput : IInput
    {
        internal enum Styles { SingleEnded, Differential };

        private List<Range> _supportedRanges = new List<Range>();
        private Device _device;
        private uint _interfaceId;
        private Styles _style;
        private int _index;
        private int _minInterChannelDelay;
        private string _name;

        internal uint InterfaceId { get { return _interfaceId; } }
        internal Styles Style { get { return _style; } }
        internal int Index { get { return _index; } }

        internal AnalogInput(Device device, uint interfaceId, Styles style, int index)
        {
            _device = device;
            _interfaceId = interfaceId;
            _style = style;
            _index = index;
            _name = SystemConnector.GetPeripheralName(device.Handle, interfaceId);
            for (int i = 0;; ++i) 
            {
                uint rangeId = SystemConnector.EnumerateRanges(device.Handle, interfaceId, i);
                if (rangeId == SystemConnector.CeboMsrRangeInvalid)
                    break;

                IList<float> minMax = SystemConnector.GetRangeMinMax(device.Handle, interfaceId, rangeId);
                _supportedRanges.Add(new Range(rangeId, minMax[0], minMax[1],
                    SystemConnector.GetDefaultAnalogInputInterChannelDelay(device.Handle, interfaceId, rangeId)));
            }
            _minInterChannelDelay = SystemConnector.GetMinAnalogInputInterChannelDelay(device.Handle, interfaceId);
        }

        /// <summary>
        /// List of the supported ranges.
        /// </summary>
        public IList<Range> SupportedRanges { get { return _supportedRanges.AsReadOnly(); } }

        /// <summary>
        /// Request default interchannel delay at the given range setting.
        /// </summary>
        /// <param name="range">Range for which this value is requested.</param>
        public int GetDefaultInterChannelDelay(Range range)
        {
            return range.DefaultInterChannelDelay;
        }

        /// <summary>
        /// Minimum interchannel delay.
        /// </summary>
        public int MinInterChannelDelay { get { return _minInterChannelDelay; } }

        /// <summary>
        /// Set new range, interchannel delay is set to the default for this range.
        /// </summary>
        /// <param name="range">Range level to set.</param>
        public void SetParameters(Range range)
        {
            SystemConnector.SetAnalogInputParameters(_device.Handle, _interfaceId, range.InterfaceId, -1);
        }

        /// <summary>
        /// Set range and user specified interchannel delay.
        /// </summary>
        /// <param name="range">Range level to set.</param>
        /// <param name="interChannelDelay">Interchannel delay in microseconds.</param>
        /// <returns>Corrected interchannel delay in microseconds.</returns>
        public int SetParameters(Range range, int interChannelDelay)
        {
            return SystemConnector.SetAnalogInputParameters(_device.Handle, _interfaceId, range.InterfaceId, interChannelDelay);
        }

        /// <summary>
        /// Define (identical to SetParameters(Range)) or read active range setting.
        /// </summary>
        public Range Range
        {
            get 
            { 
                int dummy = 0; 
                uint rangeId = SystemConnector.GetAnalogInputParameters(_device.Handle, _interfaceId, ref dummy);
                return _supportedRanges.Find(r => r.InterfaceId == rangeId);
            }
            set { SetParameters(value); }
        }

        /// <summary>
        /// Active interchannel delay.
        /// </summary>
        public int InterChannelDelay
        {
            get { int icd = 0; SystemConnector.GetAnalogInputParameters(_device.Handle, _interfaceId, ref icd); return icd; }
        }

        /// <summary>
        /// Read current voltage value.
        /// </summary>
        public float Value { get { return SystemConnector.ReadAnalogInput(_device.Handle, _interfaceId); } }

        /// <summary>
        /// Name of the component.
        /// </summary>
        public string Name { get { return _name; } }

        public override string ToString()
        {
            return _style == Styles.Differential ? ("Differential" + _index) : ("SingleEnded" + _index);
        }
    }
}
