#!/usr/bin/env python
from CeboMsrApiPython import LibraryInterface, DeviceType

# CeboMsr - Python example - Streaming Input.
#
# Multiple inputs are read multiple times at 300 Hz
# and printed to the console.
#
# @author Thomas Hoppe

def streamingInputTestLC(device):
    """
    Read blocks of data from device.
    device: Device to use for test.
    """
    # Construct selection that contains inputs to read from.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDifferentialInputs()[1],
        device.getDigitalPorts()[0],
        device.getDigitalPorts()[1]
    ]

    # Prepare device with this collection ...
    device.setupInputFrame(inputs)

    # Start sampling ...
    device.startBufferedDataAcquisition(300, 20 * 25, False)

    # Read 20 x 25 frames using blocked read,
    # this function returns after *all* (25) requested frames are collected.
    for _ in range(20):
        # Read 25 frames ...
        frames = device.readBlocking(25)

        # Write out the 1st one.
        inFrame = frames[0]
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("DigitalPort #1: %d" % inFrame.getDigitalPort(1)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V, " +
            ("Differential #1: %.2f" % inFrame.getDifferential(1)) + " V")

    # Stop the DAQ.
    device.stopDataAcquisition()

def streamingInputTestStick(device):
    """
    Read blocks of data from device.
    device: Device to use for test.
    """
    # Construct selection that contains inputs to read from.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDigitalPorts()[0]
    ]

    # Prepare device with this collection ...
    device.setupInputFrame(inputs)

    # Start sampling ...
    device.startBufferedDataAcquisition(300, 20 * 25, False)

    # Read 20 x 25 frames using blocked read,
    # this function returns after *all* (25) requested frames are collected.
    for _ in range(20):
        # Read 25 frames ...
        frames = device.readBlocking(25)

        # Write out the 1st one.
        inFrame = frames[0]
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V")

    # Stop the DAQ.
    device.stopDataAcquisition()

def main():
    """
    The examples main method.
    """
    try:
        # Search for devices ...
        devices = LibraryInterface.enumerate(DeviceType.All)

        # If at least one has been found, use the first one ...
        if (len(devices) > 0):
            device = devices[0]

            # Open device, nothing can be done without doing this.
            device.open()

            # Do some streaming ...
            if (device.getDeviceType() == DeviceType.CeboLC):
                streamingInputTestLC(device)
            elif (device.getDeviceType() == DeviceType.CeboStick):
                streamingInputTestStick(device)
            else:
                raise SystemError("No supported device detected.")

            # Finalize device usage, this free's up the device, so it can be used
            # again, including other applications.
            device.close()
    except Exception as e:
        print(e)

if (__name__ == "__main__"):
    main()
